package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.data.recipes.FinishedRecipe
import net.minecraft.data.recipes.ShapedRecipeBuilder
import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.tags.ItemTags
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraft.world.item.crafting.Ingredient
import net.minecraftforge.common.Tags
import net.minecraftforge.common.Tags.Items.COBBLESTONE
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.modLocation
import java.util.function.Consumer

fun addCraftingTableRecipes(consumer: Consumer<FinishedRecipe>) {
	MatteryRecipe(MRegistry.CARGO_CRATES.item)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.CHESTS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(Tags.Items.CHESTS)
		.build(consumer)

	for ((dye, crate) in MRegistry.CARGO_CRATES.blocks) {
		ShapelessRecipeBuilder(crate, 1)
			.requires(Ingredient.of(MRegistry.CARGO_CRATES.allItems.entries.stream().filter { it.key != dye }.map { ItemStack(it.value) }))
			.requires(dye.tag)
			.unlockedBy(MRegistry.CARGO_CRATES.allItems.entries.stream().filter { it.key != dye }.map { it.value })
			.save(consumer, "${crate.registryName}_alt")
	}

	ShapelessRecipeBuilder(MItems.TRITANIUM_INGOT_BLOCK, 1)
		.requires(Ingredient.of(MItemTags.TRITANIUM_INGOTS), 9)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.save(consumer)

	ShapelessRecipeBuilder(MItems.TRITANIUM_INGOT, 9)
		.requires(Ingredient.of(MItemTags.TRITANIUM_INGOTS_STORAGE))
		.unlockedBy(MItemTags.TRITANIUM_INGOTS_STORAGE)
		.save(consumer, modLocation("tritanium_ingot_from_storage"))

	ShapelessRecipeBuilder(MItems.ENERGY_COUNTER, 1)
		.requires(MItems.ENERGY_COUNTER)
		.unlockedBy(MItems.ENERGY_COUNTER)
		.save(consumer, modLocation("energy_counter_reset"))

	MatteryRecipe(MBlocks.PLATE_PRESS)
		.row(MItems.ELECTRIC_PARTS, MItems.ENERGY_BUS, MItems.ELECTRIC_PARTS)
		.row(MItemTags.TRITANIUM_INGOTS, Items.BLAST_FURNACE, MItemTags.TRITANIUM_INGOTS)
		.row(MItemTags.PISTONS, MItemTags.TRITANIUM_INGOTS, MItemTags.PISTONS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItems.ELECTRIC_PARTS)
		.build(consumer)

	MatteryRecipe(MBlocks.PLATE_PRESS)
		.rowB(MItems.MACHINE_FRAME)
		.rowAC(MItemTags.PISTONS, MItemTags.PISTONS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItems.ELECTRIC_PARTS)
		.build(consumer, "advanced")

	MatteryRecipe(MItems.PATTERN_DRIVE_NORMAL)
		.rowAC(MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT)
		.row(MItemTags.ADVANCED_CIRCUIT, MItemTags.TRITANIUM_PLATES, MItemTags.ADVANCED_CIRCUIT)
		.rowAC(MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT)
		.unlockedBy(MItemTags.ADVANCED_CIRCUIT)
		.build(consumer)

	// Машины
	MatteryRecipe(MItems.MATTER_RECYCLER)
		.row(MItems.MATTER_CAPACITOR_PARTS, Items.HOPPER, MItemTags.BASIC_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItems.MATTER_CABLE, MItems.MATTER_IO_PORT, MItems.MATTER_CABLE)
		.unlockedBy(MItems.MATTER_CABLE)
		.build(consumer)

	// Блоки
	MatteryRecipe(MItems.MATTER_CAPACITOR_BANK)
		.row(Tags.Items.GLASS, MItemTags.IRON_PLATES, Tags.Items.GLASS)
		.row(MItemTags.IRON_PLATES, MItems.MACHINE_FRAME, MItemTags.IRON_PLATES)
		.row(MItems.MATTER_CABLE, MItems.MATTER_IO_PORT, MItems.MATTER_CABLE)
		.unlockedBy(MItems.MATTER_CABLE)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_BANK)
		.row(Tags.Items.GLASS, MItemTags.IRON_PLATES, Tags.Items.GLASS)
		.row(MItemTags.IRON_PLATES, MItems.MACHINE_FRAME, MItemTags.IRON_PLATES)
		.row(MItems.ELECTRIC_PARTS, MItems.ENERGY_BUS, MItems.ELECTRIC_PARTS)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_STABILIZER)
		.row(MItemTags.ADVANCED_CIRCUIT, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.ADVANCED_CIRCUIT)
		.row(MItems.MATTER_TRANSFORM_MATRIX, MItems.MACHINE_FRAME, MItems.MATTER_TRANSFORM_MATRIX)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_LIMITER, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_LIMITER)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.PORTABLE_GRAVITATION_STABILIZER)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_LIMITER, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_LIMITER)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_FIELD_SENSOR)
		.rowB(MItemTags.BASIC_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ELECTROMAGNET, MItemTags.TRITANIUM_PLATES)
		.rowB(MItemTags.IRON_PLATES)
		.unlockedBy(MItems.ELECTROMAGNET)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_FIELD_LIMITER)
		.row(Tags.Items.ENDER_PEARLS, MItemTags.ADVANCED_CIRCUIT, Tags.Items.ENDER_PEARLS)
		.row(MItemTags.GOLD_WIRES, MItems.QUANTUM_TRANSCEIVER, MItemTags.GOLD_WIRES)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.QUANTUM_TRANSCEIVER)
		.build(consumer)

	MatteryRecipe(MItems.BLACK_HOLE_SCANNER)
		.row(MItemTags.IRON_PLATES, Tags.Items.GLASS_PANES_COLORLESS, MItemTags.IRON_PLATES)
		.row(MItemTags.GOLD_WIRES, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.ADVANCED_CIRCUIT)
		.rowAC(Tags.Items.DUSTS_GLOWSTONE, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.PHANTOM_ATTRACTOR)
		.row(Tags.Items.DUSTS_REDSTONE, Tags.Items.GLASS_COLORLESS, Tags.Items.DUSTS_REDSTONE)
		.row(MItemTags.TRITANIUM_PLATES, MItems.QUANTUM_TRANSCEIVER, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, ItemTags.BEDS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.QUANTUM_TRANSCEIVER)
		.build(consumer)

	MatteryRecipe(MItems.QUANTUM_TRANSCEIVER, 2)
		.rowAC(MItemTags.COPPER_WIRES, MItemTags.COPPER_WIRES)
		.row(MItemTags.GOLD_WIRES, Tags.Items.ENDER_PEARLS, MItemTags.GOLD_WIRES)
		.row(MItemTags.GOLD_WIRES, MItems.ELECTROMAGNET, MItemTags.GOLD_WIRES)
		.unlockedBy(Tags.Items.ENDER_PEARLS)
		.build(consumer)

	MatteryRecipe(MItems.ELECTROMAGNET)
		.row(MItemTags.COPPER_WIRES, Tags.Items.INGOTS_IRON, MItemTags.COPPER_WIRES)
		.unlockedBy(Tags.Items.ENDER_PEARLS)
		.build(consumer)

	MatteryRecipe(MItems.ENERGY_SERVO)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ENERGY_BUS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MItems.MIRROR_COMPOUND, 3)
		.row(Tags.Items.GLASS_PANES_COLORLESS, Tags.Items.GLASS_PANES_COLORLESS, Tags.Items.GLASS_PANES_COLORLESS)
		.row(MItemTags.IRON_PLATES, MItemTags.IRON_PLATES, MItemTags.IRON_PLATES)
		.unlockedBy(MItemTags.IRON_PLATES)
		.unlockedBy(Tags.Items.GLASS_PANES_COLORLESS)
		.build(consumer)

	// броня
	MatteryRecipe(MItems.TRITANIUM_HELMET)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_PANTS)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.rowAC(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_CHESTPLATE)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_BOOTS)
		.rowAC(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// простые батарейки
	MatteryRecipe(MItems.BATTERY_CRUDE)
		.rowB(Tags.Items.DUSTS_REDSTONE)
		.rowB(Tags.Items.CROPS_POTATO)
		.rowB(Tags.Items.INGOTS_IRON)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_BASIC)
		.rowAC(Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE)
		.rowB(MItems.ELECTRIC_PARTS)
		.rowB(MItemTags.IRON_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_NORMAL)
		.rowB(MItems.ELECTRIC_PARTS)
		.row(MItemTags.COPPER_WIRES, MItemTags.IRON_PLATES, MItemTags.COPPER_WIRES)
		.row(Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_DENSE)
		.row(Tags.Items.DUSTS_REDSTONE, MItems.ENERGY_BUS, Tags.Items.DUSTS_REDSTONE)
		.row(MItemTags.GOLD_WIRES, MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.row(Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_CAPACITOR)
		.row(Tags.Items.DUSTS_REDSTONE, MItems.ENERGY_BUS, Tags.Items.DUSTS_REDSTONE)
		.row(MItemTags.GOLD_WIRES, MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.row(MItemTags.GOLD_WIRES, Tags.Items.DUSTS_REDSTONE, MItemTags.GOLD_WIRES)
		.build(consumer)

	// накопители материи
	MatteryRecipe(MItems.MATTER_CAPACITOR_DENSE)
		.row(MItems.MATTER_CAPACITOR_PARTS, Tags.Items.GLASS, MItems.MATTER_CAPACITOR_PARTS)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.ENDER_PEARLS, MItemTags.TRITANIUM_PLATES)
		.rowAC(Tags.Items.GEMS_DIAMOND, Tags.Items.GEMS_DIAMOND)
		.build(consumer)

	// станция андроида
	MatteryRecipe(MItems.ANDROID_STATION)
		.row(MItems.ELECTRIC_PARTS, MItemTags.ADVANCED_CIRCUIT, MItems.ELECTRIC_PARTS)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ELECTRIC_PARTS, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// Энерго меч
	MatteryRecipe(MItems.ENERGY_SWORD)
		.rowBC(MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.rowBC(MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.row(MItems.BATTERY_CAPACITOR, MItems.TRITANIUM_SWORD, MItemTags.ADVANCED_CIRCUIT)
		.unlockedBy(MItems.BATTERY_CAPACITOR)
		.buildEnergetic(consumer)

	// лампа
	MatteryRecipe(MItems.LABORATORY_LAMP)
		.row(MItemTags.IRON_PLATES, MItemTags.HARDENED_GLASS_PANES_COLORLESS, MItemTags.IRON_PLATES)
		.row(MItems.MIRROR, Items.GLOWSTONE, MItems.MIRROR)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.DUSTS_REDSTONE, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// апгрейд на сетку крафта
	MatteryRecipe(MItems.ExosuitUpgrades.CRAFTING_UPGRADE)
		.row(MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT)
		.row(MItemTags.CRAFTING_TABLES, MItems.QUANTUM_TRANSCEIVER, MItemTags.CRAFTING_TABLES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.build(consumer)
}
