package ru.dbotthepony.mc.otm.block.entity.matter

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.Container
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.ForgeConfigSpec
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.capabilities.ForgeCapabilities
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.ConciseBalanceValues
import ru.dbotthepony.mc.otm.block.IDroppableContainer
import ru.dbotthepony.mc.otm.block.matter.MatterBottlerBlock
import ru.dbotthepony.mc.otm.block.entity.MatteryPoweredBlockEntity
import ru.dbotthepony.mc.otm.block.entity.WorkerState
import ru.dbotthepony.mc.otm.capability.BlockEnergyStorageImpl
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler
import ru.dbotthepony.mc.otm.capability.matter.MatterDirection
import ru.dbotthepony.mc.otm.capability.matter.MatterHandlerImpl
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.container.MatteryContainerHooks
import ru.dbotthepony.mc.otm.graph.Graph6Node
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.menu.MatterBottlerMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.core.*
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MNames
import ru.dbotthepony.mc.otm.core.WriteOnce

class MatterBottlerBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	MatteryPoweredBlockEntity(MBlockEntities.MATTER_BOTTLER, p_155229_, p_155230_), IMatterGraphNode, IDroppableContainer {

	override val matterNode = Graph6Node<IMatterGraphNode>(this)
	private val resolverNode = LazyOptional.of { this }
	override val energy = WorkerEnergyStorage(this, ENERGY_VALUES)

	var isBottling: Boolean = true
		set(value) {
			field = value
			this.setChangedLight()
		}

	fun switchWorkFlow() {
		isBottling = !isBottling
		updateBlockState()
	}

	private fun updateBlockState() {
		val level = level as? ServerLevel ?: return

		var state = blockState
		val initial = if (isBottling) 0 else 3

		for (i in initial .. initial + 2) {
			val desired = !container.getItem(i).isEmpty && container.getItem(i).getCapability(MatteryCapability.MATTER).isPresent

			if (state.getValue(MatterBottlerBlock.SLOT_PROPERTIES[i - initial]) != desired) {
				state = state.setValue(MatterBottlerBlock.SLOT_PROPERTIES[i - initial], desired)
			}
		}

		if (state !== blockState) {
			level.setBlock(blockPos, state, Block.UPDATE_CLIENTS)
		}
	}

	// TODO: оно должно что то делать
	// true - continue even if empty when bottling / if full while unbottling
	// false - spit into output slot
	//private var work_behavior = true

	val container: MatteryContainer = object : MatteryContainer(this::setChangedLight, 6) {
		override fun getMaxStackSize(slot: Int): Int {
			return 1
		}

		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)
			updateBlockState()
		}
	}

	override val droppableContainer: Container
		get() = container

	val itemHandler = container.handler(object : MatteryContainerHooks {
		override fun canInsert(slot: Int, stack: ItemStack): Boolean {
			if (isBottling) {
				return slot < 3 && stack.getCapability(MatteryCapability.MATTER).isPresent
			}

			return slot >= 3 && stack.getCapability(MatteryCapability.MATTER).isPresent
		}

		override fun canExtract(slot: Int, amount: Int, stack: ItemStack): Boolean {
			if (isBottling) {
				return slot >= 3
			}

			return slot < 3
		}
	})

	val matter: MatterHandlerImpl = object : MatterHandlerImpl(this::setChangedLight, MatterDirection.BIDIRECTIONAL, ::CAPACITY) {
		override val direction: MatterDirection get() {
			return if (this@MatterBottlerBlockEntity.isBottling) MatterDirection.RECEIVE else MatterDirection.EXTRACT
		}
	}

	private var initialCapacity: ImpreciseFraction? = null
	private var lastWorkStack: ItemStack? = null

	override fun getMatterHandler(): IMatterHandler {
		return matter
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			MatterNetworkGraph.discoverFull(this, matterNode)
	}

	private var valid = true

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
		itemHandler.invalidate()
		matter.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
		itemHandler.revive()
		matter.revive()
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap == MatteryCapability.MATTER) {
				return matter.get().cast()
			}

			if (cap == ForgeCapabilities.ITEM_HANDLER) {
				return itemHandler.get().cast()
			}

			if (cap == MatteryCapability.MATTER_NODE) {
				return resolverNode.cast()
			}
		}

		return super.getCapability(cap, side)
	}

	override val defaultDisplayName: Component
		get() = MBlocks.MATTER_BOTTLER.name

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MatterBottlerMenu(containerID, inventory, this)
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt[INVENTORY_KEY] = container.serializeNBT()
		nbt[IS_BOTTLING_KEY] = isBottling
		nbt[MATTER_STORAGE_KEY] = matter.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		isBottling = nbt.getBoolean(IS_BOTTLING_KEY)
		nbt.map(MATTER_STORAGE_KEY, matter::deserializeNBT)
		nbt.map(INVENTORY_KEY, container::deserializeNBT)
	}

	fun getWorkProgress(): Float {
		val lastWorkStack = lastWorkStack ?: return 0f
		val initialCapacity = initialCapacity ?: return 0f
		val cap = lastWorkStack.getCapability(MatteryCapability.MATTER).orNull() ?: return 0f

		if (this.isBottling) {
			if (cap.maxStoredMatter - initialCapacity <= ImpreciseFraction.ZERO) {
				return 0f
			}

			return ((cap.storedMatter - initialCapacity) / (cap.maxStoredMatter - initialCapacity)).toFloat()
		}

		if (initialCapacity <= ImpreciseFraction.ZERO) {
			return 0f
		}

		return (ImpreciseFraction.ONE - cap.storedMatter / initialCapacity).toFloat()
	}

	override fun setRemoved() {
		super.setRemoved()
		matterNode.destroy(::MatterNetworkGraph)
	}

	fun tick() {
		batteryChargeLoop()

		if (isBlockedByRedstone) {
			if (blockState.getValue(WorkerState.SEMI_WORKER_STATE) !== WorkerState.IDLE) {
				level?.setBlock(
					blockPos,
					blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE),
					Block.UPDATE_CLIENTS
				)
			}

			return
		}

		var work_stack: ItemStack? = null
		var capability: IMatterHandler? = null
		val align = if (isBottling) 0 else 3
		var work_slot = -1
		val unexpectedDirection = if (isBottling) MatterDirection.EXTRACT else MatterDirection.RECEIVE

		for (i in align until align + 3) {
			val itemStack = container.getItem(i)

			if (!itemStack.isEmpty) {
				val cap = itemStack.getCapability(MatteryCapability.MATTER).orNull() ?: continue

				if (cap.direction !== unexpectedDirection) {
					if (this.isBottling && cap.missingMatter > ImpreciseFraction.ZERO || !this.isBottling && cap.storedMatter > ImpreciseFraction.ZERO) {
						work_stack = itemStack
						capability = cap
						work_slot = i
						break
					}
				}
			}
		}

		if (work_stack == null) {
			lastWorkStack = null
			initialCapacity = null
		} else if (work_stack != lastWorkStack) {
			lastWorkStack = work_stack
			initialCapacity = capability!!.storedMatter
		}

		val graph = matterNode.graph as MatterNetworkGraph?

		if (capability != null) {
			if (blockState.getValue(WorkerState.SEMI_WORKER_STATE) !== WorkerState.WORKING) {
				level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING), Block.UPDATE_CLIENTS)
			}

			if (isBottling) {
				if (matter.storedMatter < MATTER_EXCHANGE_RATE && graph != null) {
					val extracted = graph.extractMatter(
						matter.missingMatter.coerceAtMost(MATTER_EXCHANGE_RATE * EXTRACTION_TICKS).coerceAtMost(capability.missingMatter - matter.storedMatter), true
					)

					if (extracted > ImpreciseFraction.ZERO) {
						val received = matter.receiveMatterOuter(extracted, false)
						graph.extractMatter(received, false)
					}
				}

				if (matter.storedMatter > ImpreciseFraction.ZERO) {
					val energyExtracted = energy.extractEnergyInner(ENERGY_CONSUMPTION, true)

					if (!energyExtracted.isZero) {
						val matter = capability.receiveMatterOuter(MATTER_EXCHANGE_RATE.coerceAtMost(matter.storedMatter) * energyExtracted / ENERGY_CONSUMPTION, true)

						if (!matter.isZero) {
							energy.extractEnergyInner(ENERGY_CONSUMPTION * matter / MATTER_EXCHANGE_RATE, false)

							capability.receiveMatterOuter(matter, false)
							this.matter.extractMatterInner(matter, false)

							if (capability.missingMatter.isZero) {
								for (i in 3..5) {
									if (container.getItem(i).isEmpty) {
										container.setItem(work_slot, ItemStack.EMPTY)
										container.setItem(i, work_stack!!)
										break
									}
								}
							}
						}
					}
				}
			} else {
				val energyExtracted = energy.extractEnergyInner(ENERGY_CONSUMPTION, true)

				if (!energyExtracted.isZero) {
					val matter = capability.extractMatterOuter(MATTER_EXCHANGE_RATE.coerceAtMost(matter.missingMatter) * energyExtracted / ENERGY_CONSUMPTION, true)

					if (!matter.isZero) {
						this.energy.extractEnergyInner(ENERGY_CONSUMPTION * matter / MATTER_EXCHANGE_RATE,false)

						capability.extractMatterOuter(matter, false)
						this.matter.receiveMatterInner(matter, false)

						if (capability.storedMatter.isZero) {
							for (i in 2 downTo 0) {
								if (container.getItem(i).isEmpty) {
									container.setItem(work_slot, ItemStack.EMPTY)
									container.setItem(i, work_stack!!)
									break
								}
							}
						}
					}
				}
			}
		} else {
			level!!.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE), Block.UPDATE_CLIENTS)
		}

		if (!isBottling && !matter.storedMatter.isZero && graph != null) {
			val diff = matter.extractMatterInner(matter.storedMatter, true)
			val diff2 = graph.receiveMatter(diff, true)
			matter.extractMatterInner(diff2, false)
			graph.receiveMatter(diff2, false)
		}
	}

	companion object {
		val MATTER_EXCHANGE_RATE get() = _MATTER_EXCHANGE_RATE.get()
		val ENERGY_CONSUMPTION get() = _ENERGY_CONSUMPTION.get()
		val EXTRACTION_TICKS get() = _EXTRACTION_TICKS.get()
		val CAPACITY get() = _CAPACITY.get()

		private var _MATTER_EXCHANGE_RATE: ImpreciseFractionConfigValue by WriteOnce()
		private var _ENERGY_CONSUMPTION: ImpreciseFractionConfigValue by WriteOnce()
		private var _EXTRACTION_TICKS: ImpreciseFractionConfigValue by WriteOnce()
		private var _CAPACITY: ImpreciseFractionConfigValue by WriteOnce()

		var ENERGY_VALUES: ConciseBalanceValues by WriteOnce()
			private set

		fun registerConfig(builder: ForgeConfigSpec.Builder) {
			builder.push(MNames.MATTER_BOTTLER)

			ENERGY_VALUES = BlockEnergyStorageImpl.makeConfigEntry(builder)

			_MATTER_EXCHANGE_RATE = builder.defineImpreciseFraction("matterExchangeRate", ImpreciseFraction("0.04"), ImpreciseFraction("0.0001"))
			_ENERGY_CONSUMPTION = builder.defineImpreciseFraction("energyConsumption", ImpreciseFraction(20), ImpreciseFraction.ONE)
			_EXTRACTION_TICKS = builder.defineImpreciseFraction("extractionTicks", ImpreciseFraction(200), ImpreciseFraction.ONE)
			_CAPACITY = builder.defineImpreciseFraction("matterCapacity", ImpreciseFraction(4), ImpreciseFraction.ONE_TENTH)

			builder.pop()
		}

		const val IS_BOTTLING_KEY = "isBottling"
	}
}
