package ru.dbotthepony.mc.otm.client.screen.panels.buttons

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.gui.screens.Screen
import net.minecraft.network.chat.Component
import ru.dbotthepony.mc.otm.client.playGuiClickSound
import ru.dbotthepony.mc.otm.client.render.TextAlign
import ru.dbotthepony.mc.otm.client.render.Widgets18
import ru.dbotthepony.mc.otm.client.render.drawAligned
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel
import ru.dbotthepony.mc.otm.core.RGBAColor

open class ButtonPanel<out S : Screen>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = 40f,
	height: Float = HEIGHT,
	var label: Component
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	constructor(screen: S, parent: EditablePanel<*>?, label: Component) : this(screen, parent, x = 0f, label = label)

	constructor(
		screen: S,
		parent: EditablePanel<*>?,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = 40f,
		height: Float = HEIGHT,
		label: Component,
		onPress: Runnable
	) : this(screen, parent, x, y, width, height, label) {
		this.callback = onPress
	}

	protected var callback: Runnable? = null
	protected var pressed = false

	protected open fun onClick(button: Int) {
		callback?.run()
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		if (isDisabled || pressed) {
			return true
		}

		if (!tryToGrabMouseInput()) {
			return true
		}

		pressed = true
		playGuiClickSound()
		return true
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		if (!pressed) {
			return true
		}

		grabMouseInput = false
		pressed = false

		if (isHovered) {
			onClick(button)
		}

		return true
	}

	fun bind(runnable: Runnable) {
		callback = runnable
	}

	var textColor = RGBAColor.WHITE

	var isDisabled = false
		set(value) {
			if (field != value) {
				if (!value) {
					pressed = false
					grabMouseInput = false
				}

				field = value
			}
		}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		if (isDisabled) {
			Widgets18.BUTTON_DISABLED_STRETCHABLE.render(stack, width = width, height = height)
		} else if (pressed) {
			Widgets18.BUTTON_PRESSED_STRETCHABLE.render(stack, width = width, height = height)
		} else if (isHovered) {
			Widgets18.BUTTON_HOVERED_STRETCHABLE.render(stack, width = width, height = height)
		} else {
			Widgets18.BUTTON_IDLE_STRETCHABLE.render(stack, width = width, height = height)
		}

		font.drawAligned(stack, label, TextAlign.CENTER_CENTER, width / 2f, height / 2f, textColor.toInt())
	}

	override fun sizeToContents() {
		super.sizeToContents()

		height = height.coerceAtLeast(HEIGHT).coerceAtLeast(font.lineHeight.toFloat() + 2f)
		width = width.coerceAtLeast(HEIGHT).coerceAtLeast(font.width(label) + 4f)
	}

	companion object {
		const val HEIGHT = 18f
	}
}
