package ru.dbotthepony.mc.otm.graph

import java.util.*
import kotlin.collections.ArrayList

class GraphFlooder<T>(val startNode: Graph6Node<T>, @JvmField val seen: Int = nextSeen++) {
	var flooded = false
		private set

	private val _nodes = ArrayList<Graph6Node<T>>()

	@JvmField
	val nodes = Collections.unmodifiableCollection(_nodes)!!

	fun size() = _nodes.size

	private fun flood(node: Graph6Node<T>) {
		if (node.seen >= seen) return

		_nodes.add(node)
		node.seen = seen

		if (node.top != null) flood(node.top!!)
		if (node.bottom != null) flood(node.bottom!!)
		if (node.left != null) flood(node.left!!)
		if (node.right != null) flood(node.right!!)
		if (node.front != null) flood(node.front!!)
		if (node.back != null) flood(node.back!!)
	}

	fun flood() {
		if (flooded) throw IllegalStateException("Already flooded")

		flooded = true

		_nodes.add(startNode)
		startNode.seen = seen

		if (startNode.top != null) flood(startNode.top!!)
		if (startNode.bottom != null) flood(startNode.bottom!!)
		if (startNode.left != null) flood(startNode.left!!)
		if (startNode.right != null) flood(startNode.right!!)
		if (startNode.front != null) flood(startNode.front!!)
		if (startNode.back != null) flood(startNode.back!!)
	}

	companion object {
		private var nextSeen = 0

		fun <T> floodIf(node: Graph6Node<T>?, seen: Int, runnable: (GraphFlooder<T>) -> Unit) {
			if (node != null && node.seen < seen) {
				if (seen == Int.MAX_VALUE) {
					val flooder = GraphFlooder(node)
					flooder.flood()
					runnable(flooder)
				} else {
					val flooder = GraphFlooder(node, seen)
					flooder.flood()
					runnable(flooder)
				}
			}
		}
	}
}
